# -*- coding: utf-8 -*-
from snowflake.connector import connect
from sqlalchemy import create_engine
import pandas
from abc import ABC, abstractmethod
import json

class MPPDatabaseEngine(ABC):

    @abstractmethod
    def Connect(self):
        pass

    @abstractmethod
    def Disconnect(self):
        pass

    @abstractmethod
    def DMLQuery(self):
        pass

    @abstractmethod
    def SelectQuery(self):
        pass

    def __LoadConfigurationFileFromLocalJSON(self , ConfigurationFilePath):
        with open(ConfigurationFilePath ,'r') as ConfigFile:
            return json.load(ConfigFile , strict = False)


class SnowflakeClient(MPPDatabaseEngine): # Pending to allow for keypar auth

    ActiveSnowflakeCursor = None
    SQLAlchemyConnection = None


    def __init__(self , Config):
        if isinstance(Config,str):
            self.ConfigurationDict = self._MPPDatabaseEngine__LoadConfigurationFileFromLocalJSON(Config)
        elif isinstance(Config,dict):
            self.ConfigurationDict = Config

        self.__SetSnowflakeConnectorContext()
        self.__SetSQLAlchemyEngine()

    def __SetSnowflakeConnectorContext(self):
        self.ActiveContext = connect(user = self.ConfigurationDict["user"],
                                     password = self.ConfigurationDict["password"],
                                     account = self.ConfigurationDict["account"]
                                     )
    def __SetSQLAlchemyEngine(self):
        self.SQLAlchemyEngine = create_engine('snowflake://{user}:{password}@{account}/'.format(
                                                user = self.ConfigurationDict["user"],
                                                password = self.ConfigurationDict["password"],
                                                account = self.ConfigurationDict["account"],
                                                )
                                             )

    def Connect(self):
        self.ActiveSnowflakeCursor = self.ActiveContext.cursor()
        self.SQLAlchemyConnection = self.SQLAlchemyEngine.connect()
        print('Snowflake Connection Open')

    def Disconnect(self):
        self.ActiveSnowflakeCursor.close()
        self.SQLAlchemyConnection.close()
        print('Snowflake Connection Closed')

    def SwitchTo(self , Object , ObjectName):
        if Object.upper() == "WAREHOUSE":
            self.__ResumeWarehouseIfSuspended(ObjectName)
        SwitchingQuery = "USE {OBJECT} {OBJECT_NAME};".format(OBJECT = Object , OBJECT_NAME = ObjectName)
        self.ActiveSnowflakeCursor.execute(SwitchingQuery)
        self.SQLAlchemyConnection.execute(SwitchingQuery)
        print("Switched to {OBJECT} = {OBJECT_NAME} successfully".format(OBJECT = Object , OBJECT_NAME = ObjectName))

    def __CheckWarehouseState(self , Warehouse):
        Warehouse = Warehouse.upper()
        WarehouseStateDataFrame = self.SelectQuery("SHOW WAREHOUSES LIKE '{WAREHOUSE}';".format(WAREHOUSE = Warehouse))
        WarehouseState = WarehouseStateDataFrame.loc[WarehouseStateDataFrame["name"] == Warehouse , "state"].values[0]
        return WarehouseState

    def __ResumeWarehouseIfSuspended(self , Warehouse):
        WarehouseState = self.__CheckWarehouseState(Warehouse)
        if WarehouseState == "SUSPENDED":
            self.DMLQuery("ALTER WAREHOUSE {WAREHOUSE} resume;".format(WAREHOUSE = Warehouse))

    def SuspendWarehouse(self , Warehouse):
        self.DMLQuery("ALTER WAREHOUSE {WAREHOUSE} suspend".format(WAREHOUSE = Warehouse))
        print("Warehouse {WAREHOUSE} suspended.".format(WAREHOUSE = Warehouse))

    def DMLQuery(self , QueryString):
        self.ActiveSnowflakeCursor.execute(QueryString)

    def SelectQuery(self,QueryString):
        return pandas.read_sql(QueryString,
                               con = self.SQLAlchemyConnection)

    def AlterWarehouse(self , Warehouse , Action):
        Status = self.__CheckWarehouseStatus(Warehouse)
        if Action == 'suspend' and Status != 'SUSPENDED':
            self.DMLQuery('Alter warehouse {warehouse} suspend;'.format(warehouse = Warehouse))
        elif Action == 'resume' and Status in ('SUSPENDED','SUSPENDING'):
            self.DMLQuery('Alter warehouse {warehouse} resume;'.format(warehouse = Warehouse))

    def __CheckWarehouseStatus(self , Warehouse):
        WHmeta = self.SelectQuery(QueryString = "show warehouses like '{warehouse}';".format(warehouse = Warehouse))
        return WHmeta.loc[0,'state']
